==> ideaIC-213.7172.25-sources.jar: com/android/tools/idea/gradle/dsl/model/GradleDslBlockModel.java <==
/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.gradle.dsl.model;

import static com.android.tools.idea.gradle.dsl.model.ext.PropertyUtil.removeElement;

import com.android.tools.idea.gradle.dsl.api.ext.GradlePropertyModel;
import com.android.tools.idea.gradle.dsl.api.ext.PasswordPropertyModel;
import com.android.tools.idea.gradle.dsl.api.ext.ResolvedPropertyModel;
import com.android.tools.idea.gradle.dsl.api.java.LanguageLevelPropertyModel;
import com.android.tools.idea.gradle.dsl.api.util.GradleDslModel;
import com.android.tools.idea.gradle.dsl.model.ext.GradlePropertyModelBuilder;
import com.android.tools.idea.gradle.dsl.model.ext.GradlePropertyModelImpl;
import com.android.tools.idea.gradle.dsl.model.ext.PropertyUtil;
import com.android.tools.idea.gradle.dsl.parser.elements.GradleDslElement;
import com.android.tools.idea.gradle.dsl.parser.elements.GradleDslExpression;
import com.android.tools.idea.gradle.dsl.parser.elements.GradlePropertiesDslElement;
import com.android.tools.idea.gradle.dsl.parser.semantics.ModelPropertyDescription;
import com.intellij.psi.PsiElement;
import com.intellij.util.containers.HashSetQueue;
import java.util.HashSet;
import java.util.Queue;
import java.util.Set;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Base class for the models representing block elements.
 */
public abstract class GradleDslBlockModel implements GradleDslModel {
  protected GradlePropertiesDslElement myDslElement;

  protected GradleDslBlockModel(@NotNull GradlePropertiesDslElement dslElement) {
    myDslElement = dslElement;
  }

  @Override
  @Nullable
  public PsiElement getPsiElement() {
    return myDslElement.getPsiElement();
  }

  public boolean hasValidPsiElement() {
    PsiElement psiElement = getPsiElement();
    return psiElement != null && psiElement.isValid();
  }

  @Override
  @NotNull
  public Map<String, GradlePropertyModel> getInScopeProperties() {
    return myDslElement.getInScopeElements().entrySet().stream()
                       .collect(Collectors.toMap(e -> e.getKey(), e -> new GradlePropertyModelImpl(e.getValue())));
  }

  @Override
  @NotNull
  public List<GradlePropertyModel> getDeclaredProperties() {
    return myDslElement.getContainedElements(true).stream()
                       .filter(e -> e instanceof GradleDslExpression)
                       .map(e -> new GradlePropertyModelImpl(e)).collect(Collectors.toList());
  }

  @Override
  @Nullable
  public PsiElement getRepresentativeContainedPsiElement() {
    PsiElement psiElement = getPsiElement();
    if (psiElement != null) return psiElement;
    Queue<GradleDslElement> elementQueue = new HashSetQueue<>();
    Set<GradleDslElement> visitedSet = new HashSet<>();
    visitedSet.add(myDslElement);
    for (GradleDslElement newElement : myDslElement.getOriginalElements()) {
      if (!visitedSet.contains(newElement)) {
        elementQueue.add(newElement);
      }
    }
    while (!elementQueue.isEmpty()) {
      GradleDslElement element = elementQueue.remove();
      psiElement = element.getPsiElement();
      if (psiElement != null) return psiElement;
      visitedSet.add(element);
      if (element instanceof GradlePropertiesDslElement) {
        for (GradleDslElement newElement : ((GradlePropertiesDslElement)element).getOriginalElements()) {
          if (!visitedSet.contains(newElement)) {
            elementQueue.add(newElement);
          }
        }
      }
    }
    return null;
  }

  @Override
  public void delete() {
    removeElement(myDslElement);
  }

  @NotNull
  protected ResolvedPropertyModel getModelForProperty(@NotNull String property) {
    return GradlePropertyModelBuilder.create(myDslElement, property).buildResolved();
  }

  @NotNull
  protected ResolvedPropertyModel getModelForProperty(@NotNull ModelPropertyDescription property) {
    return GradlePropertyModelBuilder.create(myDslElement, property).buildResolved();
  }

  @NotNull
  protected LanguageLevelPropertyModel getLanguageModelForProperty(@NotNull String property) {
    return GradlePropertyModelBuilder.create(myDslElement, property).buildLanguage();
  }

  @NotNull
  protected LanguageLevelPropertyModel getJvmTargetModelForProperty(@NotNull String property) {
    return GradlePropertyModelBuilder.create(myDslElement, property).buildJvmTarget();
  }

  @NotNull
  protected ResolvedPropertyModel getFileModelForProperty(@NotNull String property) {
    return GradlePropertyModelBuilder.create(myDslElement, property).addTransform(PropertyUtil.FILE_TRANSFORM).buildResolved();
  }

  @NotNull
  protected PasswordPropertyModel getPasswordModelForProperty(@NotNull String property) {
    return GradlePropertyModelBuilder.create(myDslElement, property).buildPassword();
  }
}

==> ideaIC-2017.2.6-sources.jar: com/android/tools/idea/gradle/dsl/model/GradleDslBlockModel.java <==
/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.android.tools.idea.gradle.dsl.model;

import com.android.tools.idea.gradle.dsl.model.values.GradleNotNullValue;
import com.android.tools.idea.gradle.dsl.model.values.GradleNullableValue;
import com.android.tools.idea.gradle.dsl.parser.elements.GradleDslElement;
import com.android.tools.idea.gradle.dsl.parser.elements.GradlePropertiesDslElement;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.plugins.groovy.lang.psi.GroovyPsiElement;

/**
 * Base class for the models representing block elements.
 */
public abstract class GradleDslBlockModel {
  protected GradlePropertiesDslElement myDslElement;

  protected GradleDslBlockModel(@NotNull GradlePropertiesDslElement dslElement) {
    myDslElement = dslElement;
  }

  @Nullable
  public GroovyPsiElement getPsiElement() {
    return myDslElement.getPsiElement();
  }

  public boolean hasValidPsiElement() {
    GroovyPsiElement psiElement = getPsiElement();
    return psiElement != null && psiElement.isValid();
  }

  @NotNull
  protected GradleNullableValue<String> getIntOrStringValue(@NotNull String propertyName) {
    Integer intValue = myDslElement.getLiteralProperty(propertyName, Integer.class).value();
    if (intValue != null) {
      GradleDslElement propertyElement = myDslElement.getPropertyElement(propertyName);
      assert propertyElement != null;
      return new GradleNotNullValue<>(propertyElement, intValue.toString());
    }
    return myDslElement.getLiteralProperty(propertyName, String.class);
  }
}
